# -*- mode: python -*-
# -*- coding: utf-8 -*-

"""Test de base de ScoDoc

Création 10 étudiants, formation, semestre, inscription etudiant, creation 1 evaluation, saisie 10 notes.

Utiliser comme: 
     scotests/scointeractive.sh -r TEST00 scotests/test_basic.py

"""
import random

# La variable context est définie par le script de lancement
# l'affecte ainsi pour évietr les warnins pylint:
context = context  # pylint: disable=undefined-variable
REQUEST = REQUEST  # pylint: disable=undefined-variable
import scotests.sco_fake_gen as sco_fake_gen  # pylint: disable=import-error
import sco_utils
import sco_abs
import sco_abs_views
import sco_bulletins
import sco_evaluations
import sco_codes_parcours
import sco_parcours_dut
import sco_formsemestre_validation

G = sco_fake_gen.ScoFake(context.Notes)
G.verbose = False

# --- Création d'étudiants
etuds = [G.create_etud(code_nip=None) for _ in range(10)]

# --- Création d'une formation
f = G.create_formation(acronyme="")
ue = G.create_ue(formation_id=f["formation_id"], acronyme="TST1", titre="ue test")
mat = G.create_matiere(ue_id=ue["ue_id"], titre="matière test")
mod = G.create_module(
    matiere_id=mat["matiere_id"],
    code="TSM1",
    coefficient=1.0,
    titre="module test",
    ue_id=ue["ue_id"],  # faiblesse de l'API
    formation_id=f["formation_id"],  # faiblesse de l'API
)

# --- Mise place d'un semestre
sem = G.create_formsemestre(
    formation_id=f["formation_id"],
    semestre_id=1,
    date_debut="01/01/2020",
    date_fin="30/06/2020",
)

mi = G.create_moduleimpl(
    module_id=mod["module_id"],
    formsemestre_id=sem["formsemestre_id"],
    responsable_id="bach",
)

# --- Inscription des étudiants
for etud in etuds:
    G.inscrit_etudiant(sem, etud)

# --- Creation évaluation
e = G.create_evaluation(
    moduleimpl_id=mi["moduleimpl_id"],
    jour="01/01/2020",
    description="evaluation test",
    coefficient=1.0,
)

# --- Saisie toutes les notes de l'évaluation
for etud in etuds:
    nb_changed, nb_suppress, existing_decisions = G.create_note(
        evaluation=e, etud=etud, note=float(random.randint(0, 20))
    )

# --- Vérifie que les notes sont prises en compte:
b = sco_bulletins.formsemestre_bulletinetud_dict(
    context.Notes, sem["formsemestre_id"], etud["etudid"], REQUEST=REQUEST
)
# Toute les notes sont saisies, donc eval complète
etat = sco_evaluations.do_evaluation_etat(context.Notes, e["evaluation_id"])
assert etat["evalcomplete"]
# Un seul module, donc moy gen == note module
assert b["ues"][0]["cur_moy_ue_txt"] == b["ues"][0]["modules"][0]["mod_moy_txt"]
# Note au module égale à celle de l'éval
assert (
    b["ues"][0]["modules"][0]["mod_moy_txt"]
    == b["ues"][0]["modules"][0]["evaluations"][0]["note_txt"]
)


# --- Une autre évaluation
e2 = G.create_evaluation(
    moduleimpl_id=mi["moduleimpl_id"],
    jour="02/01/2020",
    description="evaluation test 2",
    coefficient=1.0,
)
# Saisie les notes des 5 premiers étudiants:
for etud in etuds[:5]:
    nb_changed, nb_suppress, existing_decisions = G.create_note(
        evaluation=e2, etud=etud, note=float(random.randint(0, 20))
    )
# Cette éval n'est pas complète
etat = sco_evaluations.do_evaluation_etat(context.Notes, e2["evaluation_id"])
assert etat["evalcomplete"] == False
# la première éval est toujours complète:
etat = sco_evaluations.do_evaluation_etat(context.Notes, e["evaluation_id"])
assert etat["evalcomplete"]

# Modifie l'évaluation 2 pour "prise en compte immédiate"
e2["publish_incomplete"] = "1"
context.Notes.do_evaluation_edit(REQUEST, e2)
etat = sco_evaluations.do_evaluation_etat(context.Notes, e2["evaluation_id"])
assert etat["evalcomplete"] == False
assert etat["nb_att"] == 0  # il n'y a pas de notes (explicitement) en attente
assert etat["evalattente"]  # mais l'eval est en attente (prise en compte immédiate)

# Saisie des notes qui manquent:
for etud in etuds[5:]:
    nb_changed, nb_suppress, existing_decisions = G.create_note(
        evaluation=e2, etud=etud, note=float(random.randint(0, 20))
    )
etat = sco_evaluations.do_evaluation_etat(context.Notes, e2["evaluation_id"])
assert etat["evalcomplete"]
assert etat["nb_att"] == 0
assert not etat["evalattente"]  # toutes les notes sont présentes

# --- Saisie absences
etudid = etuds[0]["etudid"]

_ = sco_abs_views.doSignaleAbsence(
    context.Absences,
    "15/01/2020",
    "18/01/2020",
    demijournee=2,
    etudid=etudid,
    REQUEST=REQUEST,
)

_ = sco_abs_views.doJustifAbsence(
    context.Absences,
    "17/01/2020",
    "18/01/2020",
    demijournee=2,
    etudid=etudid,
    REQUEST=REQUEST,
)

a = sco_abs.getAbsSemEtud(context.Absences, sem, etudid)
assert a.CountAbs() == 3
assert a.CountAbsJust() == 1

# --- Permission saisie notes et décisions de jury, avec ou sans démission ou défaillance
# on n'a pas encore saisi de décisions
assert not sco_parcours_dut.formsemestre_has_decisions(context, sem["formsemestre_id"])
# Saisie d'un décision AJ, non assidu
etudid = etuds[-1]["etudid"]
sco_parcours_dut.formsemestre_validate_ues(
    context.Notes,
    sem["formsemestre_id"],
    etudid,
    sco_codes_parcours.AJ,
    False,
    REQUEST=REQUEST,
)
assert sco_parcours_dut.formsemestre_has_decisions(
    context.Notes, sem["formsemestre_id"]
)
# Suppression de la décision
sco_formsemestre_validation.formsemestre_validation_suppress_etud(
    context.Notes, sem["formsemestre_id"], etudid
)
assert not sco_parcours_dut.formsemestre_has_decisions(
    context.Notes, sem["formsemestre_id"]
)
